@@ -5,21 +5,20 @@ import re
 import sys
 from collections import defaultdict
 from contextlib import contextmanager
-from itertools import count
-from types import FunctionType, SimpleNamespace
+from types import FunctionType, MethodType, SimpleNamespace
 
 from ptera import (
-    ABSENT,
     BaseOverlay,
-    PteraFunction,
+    Immediate,
+    PteraNameError,
     Tag,
     TagSet,
+    is_tooled,
     match_tag,
     select,
     tag,
     tooled,
 )
-from ptera.selfless import PreState, PteraNameError
 
 from .config import ConfigFile
 
@@ -27,7 +26,29 @@ Option = tag.Option
 Argument = Option
 
 
-_count = count()
+def through_method(x):
+    if isinstance(x, MethodType):
+        return x.__func__
+    else:
+        return x
+
+
+def get_extras(x):
+    extras = getattr(through_method(x), "__coleo_extras__", [])
+    if isinstance(x, MethodType):
+        extras += getattr(x.__self__, "__coleo_extras__", [])
+    return extras
+
+
+class ConflictError(Exception):
+    pass
+
+
+def default(x):
+    # Normally the value of an Option should be guarded with default()
+    # in order to make it clearer that the value can be overriden, but
+    # this currently doesn't do anything.
+    return x
 
 
 def _catalogue(seen, results, fn):
@@ -35,15 +56,16 @@ def _catalogue(seen, results, fn):
         return
 
     seen.add(id(fn))
+    thru = through_method(fn)
 
-    if isinstance(fn, PteraFunction):
-        state = fn.state.state if isinstance(fn.state, PreState) else fn.state
-        tst = type(state)
-        res = tst.__info__
-        results[fn] = res
-        for name, ann in res.items():
-            val = getattr(state, name, ABSENT)
-            _catalogue(seen, results, val)
+    if is_tooled(thru):
+        info = thru.__ptera_info__
+        glb = thru.__globals__
+        results[fn] = info
+        for name, props in info.items():
+            if props["provenance"] == "external":
+                if name in glb:
+                    _catalogue(seen, results, glb[name])
 
     elif isinstance(fn, (list, tuple)):
         for entry in fn:
@@ -179,9 +201,7 @@ class Configurator:
         *,
         argparser,
         entry_point=None,
-        extras=[],
         tag=Argument,
-        description=None,
         eval_env=None,
         expand=None,
     ):
@@ -278,6 +298,8 @@ class Configurator:
                         opts.nargs = argparse.REMAINDER
                     elif command in ["nargs", "positional"]:
                         opts.positional = command == "positional"
+                        if arg == "--" or arg == "**":
+                            arg = argparse.REMAINDER
                         opts.nargs = arg or None
                         try:
                             opts.nargs = int(opts.nargs)
@@ -425,9 +447,11 @@ class Configurator:
     @contextmanager
     def __call__(self, argv=None):
         opts = parse_options(self.argparser, argv=argv, expand=self.expand)
-        opts = {k: v for k, v in vars(opts).items() if not k.startswith("#")}
-        with _setvars(opts, self.tag):
-            yield opts
+        values = {k: v for k, v in vars(opts).items() if not k.startswith("#")}
+        values["__argparser__"] = self.argparser
+        values["__options__"] = opts
+        with _setvars(values, self.tag):
+            yield values
 
 
 def _getdoc(obj):
@@ -451,6 +475,11 @@ def _make_cli_helper(parser, entry, extras, **kwargs):
             )
             _make_cli_helper(subparser, subentry, extras, **kwargs)
 
+    elif hasattr(entry, "__coleo_structure__"):
+        return _make_cli_helper(
+            parser, entry.__coleo_structure__, extras, **kwargs
+        )
+
     elif inspect.isclass(entry):
         structure = {"__doc__": entry.__doc__}
         for name, entry2 in vars(entry).items():
@@ -458,31 +487,45 @@ def _make_cli_helper(parser, entry, extras, **kwargs):
                 entry2 = tooled(entry2)
                 setattr(entry, name, entry2)
                 structure[name] = entry2
-            elif isinstance(entry2, (PteraFunction, dict, type)):
+            elif is_tooled(through_method(entry2)) or isinstance(
+                entry2, (dict, type)
+            ):
                 structure[name] = entry2
         return _make_cli_helper(parser, structure, extras, **kwargs)
 
     else:
         if isinstance(entry, FunctionType):
             entry = tooled(entry)
-        if not isinstance(entry, PteraFunction):
+        if not is_tooled(through_method(entry)):
             raise TypeError(
-                f"Expected a class, dict a function, not {type(entry)}"
+                f"Expected a class, dict or a tooled function, not {type(entry)}"
             )
-        all_entries = [entry, *extras, getattr(entry, "coleo_extras", [])]
+        all_entries = [
+            entry,
+            *extras,
+            *get_extras(entry),
+        ]
         cfg = Configurator(entry_point=all_entries, argparser=parser, **kwargs)
         parser.set_defaults(**{"#cfg": (cfg, entry)})
 
 
 def _setvars(values, tag):
-    def _resolver(value):
-        return lambda **_: value
+    def _intercepter(value):
+        def intercept(args):
+            return value
+
+        return intercept
 
     return BaseOverlay(
-        {
-            select(f"{name}:##X", env={"##X": tag}): {"value": _resolver(value)}
+        *[
+            Immediate(
+                select(f"{name}:##X", env={"##X": tag})
+                if not name.startswith("__")
+                else select(name),
+                intercept=_intercepter(value),
+            )
             for name, value in values.items()
-        }
+        ],
     )
 
 
@@ -499,6 +542,7 @@ def make_cli(
     description=None,
     eval_env=None,
     expand=None,
+    parser=None,
 ):
     """Create a coleo CLI from a function, dict or class.
 
@@ -517,6 +561,7 @@ def make_cli(
             provided, a sequence of options such as `--opt :x` will try to
             resolve `eval_env["x"]`.
         expand: An ArgsExpander to use to fill in all the arguments.
+        parser: An existing ArgumentParser instance to extend.
 
     Returns:
         The tuple (opts, call) such that the command-line application can be
@@ -525,7 +570,7 @@ def make_cli(
     if expand is None or isinstance(expand, str):
         expand = ArgsExpander(prefix=expand or "", default_file=None)
 
-    parser = argparse.ArgumentParser(
+    parser = parser or argparse.ArgumentParser(
         description=description or _getdoc(entry),
         argument_default=argparse.SUPPRESS,
     )
@@ -616,9 +661,8 @@ def auto_cli(entry, args=(), **kwargs):  # pragma: no cover
 
 def with_extras(*extras):
     def deco(fn):
-        if not isinstance(fn, PteraFunction):
-            fn = tooled(fn)
-        fn.coleo_extras = extras
+        fn = tooled(fn)
+        fn.__coleo_extras__ = extras
         return fn
 
     return deco
