@@ -28,9 +28,7 @@ def formatter_yaml():
     try:
         import yaml
     except ModuleNotFoundError:  # pragma: no cover
-        raise OSError(
-            "Please install the 'pyyaml' module to read '.yaml' files"
-        )
+        raise OSError("Please install 'coleo[yaml]' to read '.yaml' files")
     return {
         "load": yaml.full_load,
         "dump": yaml.dump,
@@ -42,7 +40,7 @@ def formatter_toml():
     try:
         import toml
     except ModuleNotFoundError:  # pragma: no cover
-        raise OSError("Please install the 'toml' module to read '.toml' files")
+        raise OSError("Please install 'coleo[toml]' to read '.toml' files")
     return {
         "load": toml.load,
         "dump": toml.dump,
